"""An exception class representing a language error condition."""
from attr import attrib, attrs, validators
from typing import Optional


@attrs(slots=True)
class LanguageError(Exception):
    """A base class representing a language error condition."""
    message: str = attrib(validator=validators.instance_of(str))
    location: Optional[str] = attrib(validator=validators.optional(validators.instance_of(str)))



from unittest import TestCase
from typing import Tuple
from click.testing import CliRunner
from aac.execute.command_line import cli, initialize_cli
from aac.execute.aac_execution_result import ExecutionStatus
from aac.context.language_context import LanguageContext


from aac.plugins.no_ext_for_final.no_ext_for_final_impl import plugin_name


from aac.plugins.no_ext_for_final.no_ext_for_final_impl import no_extension_for_final


class TestNoExtForFinal(TestCase):
    def test_no_extension_for_final(self):
        context = LanguageContext()
        schema_definition = context.get_definitions_by_name("aac.lang.Schema")
        if len(schema_definition) != 1:
            self.fail("Expected to find one and only one Schema definition")
        schema_definition = schema_definition[0]
        definitions = context.parse_and_load(root_schema_with_good_ext)
        result = no_extension_for_final(definitions[0].instance, definitions[0], schema_definition.instance)
        self.assertTrue(result.is_success())
        result = no_extension_for_final(definitions[1].instance, definitions[1], schema_definition.instance)
        self.assertTrue(result.is_success())
        context.remove_definitions(definitions)

        definitions = context.parse_and_load(root_schema_with_bad_ext)
        result = no_extension_for_final(definitions[0].instance, definitions[0], schema_definition.instance)
        self.assertTrue(result.is_success())
        result = no_extension_for_final(definitions[1].instance, definitions[1], schema_definition.instance)
        self.assertFalse(result.is_success())
        context.remove_definitions(definitions)

root_schema_with_good_ext = """
schema:
  name: TestParent
  package: test_aac.plugins.no_ext_for_final
  fields:
    - name: name
      type: string
    - name: test_field
      type: string
---
schema:
  name: TestChild
  package: test_aac.plugins.no_ext_for_final
  extends: 
    - name: TestParent
      package: test_aac.plugins.no_ext_for_final
  fields:
    - name: name
      type: string
    - name: test_field
      type: string
"""

root_schema_with_bad_ext = """
schema:
  name: TestSchema
  package: test_aac.plugins.no_ext_for_final
  modifiers:
    - final
  fields:
    - name: name
      type: string
    - name: test_field
      type: string
---
schema:
  name: TestChild
  package: test_aac.plugins.no_ext_for_final
  extends: 
    - name: TestSchema
      package: test_aac.plugins.no_ext_for_final
  fields:
    - name: name
      type: string
    - name: test_field
      type: string
"""


from unittest import TestCase
from typing import Tuple
from click.testing import CliRunner
from aac.execute.command_line import cli, initialize_cli
from aac.execute.aac_execution_result import ExecutionStatus
from aac.context.language_context import LanguageContext


from aac.plugins.unique_root_keys.unique_root_keys_impl import plugin_name


from aac.plugins.unique_root_keys.unique_root_keys_impl import root_key_names_are_unique


class TestUniqueRootKeys(TestCase):
    def test_root_key_names_are_unique(self):
        context = LanguageContext()
        definitions = context.parse_and_load(GOOD_ROOTS)
        result = root_key_names_are_unique(context)
        self.assertTrue(result.is_success())
        context.remove_definitions(definitions)

        definitions = context.parse_and_load(BAD_ROOTS)
        result = root_key_names_are_unique(context)
        self.assertFalse(result.is_success())
        context.remove_definitions(definitions)



GOOD_ROOTS = """
schema:
  name: One
  package: test.root_keys
  root: one
  fields:
    - name: name
      type: string
---
schema:
  name: Two
  package: test.root_keys
  root: two
  fields:
    - name: name
      type: string
"""

BAD_ROOTS = """
schema:
  name: One
  package: test.root_keys
  root: one
  fields:
    - name: name
      type: string
---
schema:
  name: Two
  package: test.root_keys
  root: one
  fields:
    - name: name
      type: string
"""